@@ -5,7 +5,6 @@ import re
 from typing import Annotated
 
 from bioutils.accessions import coerce_namespace
-from civicpy.civic import MolecularProfile
 from cool_seq_tool.app import CoolSeqTool
 from cool_seq_tool.schemas import CoordinateType, Strand
 from ga4gh.core import ga4gh_identify
@@ -39,6 +38,9 @@ from fusor.models import (
     Fusion,
     FusionType,
     GeneElement,
+    GenomicLocation,
+    InternalTandemDuplication,
+    InternalTandemDuplicationElements,
     LinkerElement,
     MultiplePossibleGenesElement,
     RegulatoryClass,
@@ -49,7 +51,7 @@ from fusor.models import (
     UnknownGeneElement,
 )
 from fusor.nomenclature import generate_nomenclature
-from fusor.tools import get_error_message, translate_identifier
+from fusor.tools import get_error_message
 
 _logger = logging.getLogger(__name__)
 
@@ -127,7 +129,6 @@ class FUSOR:
             categorical_attributes = any(
                 [
                     "critical_functional_domains" in kwargs,
-                    "civic_molecular_profiles" in kwargs,
                     self._contains_element_type(
                         kwargs, StructuralElementType.MULTIPLE_POSSIBLE_GENES_ELEMENT
                     ),
@@ -164,7 +165,6 @@ class FUSOR:
         regulatory_element: RegulatoryElement | None = None,
         critical_functional_domains: list[FunctionalDomain] | None = None,
         reading_frame_preserved: bool | None = None,
-        civic_molecular_profiles: list[MolecularProfile] | None = None,
     ) -> CategoricalFusion:
         """Construct a CategoricalFusion object
 
@@ -173,7 +173,6 @@ class FUSOR:
         :param critical_functional_domains: lost or preserved functional domains
         :param reading_frame_preserved: ``True`` if reading frame is preserved.
             ``False`` otherwise
-        :param civic_molecular_profiles: A list of MolecularProfile objects
         :return: CategoricalFusion if construction successful
         :raise: FUSORParametersException if given incorrect fusion properties
         """
@@ -183,7 +182,6 @@ class FUSOR:
                 criticalFunctionalDomains=critical_functional_domains,
                 readingFramePreserved=reading_frame_preserved,
                 regulatoryElement=regulatory_element,
-                civic_molecular_profiles=civic_molecular_profiles,
             )
         except ValidationError as e:
             error_message = get_error_message(e)
@@ -222,6 +220,40 @@ class FUSOR:
             raise FUSORParametersException(error_message) from e
         return fusion
 
+    @staticmethod
+    def internal_tandem_duplication(
+        structure: list[InternalTandemDuplicationElements],
+        causative_event: CausativeEvent | None = None,
+        assay: Assay | None = None,
+        regulatory_element: RegulatoryElement | None = None,
+        reading_frame_preserved: bool | None = None,
+        critical_functional_domains: list[FunctionalDomain] | None = None,
+    ) -> InternalTandemDuplication:
+        """Construct an InternalTandemDuplication (ITD) object
+
+        :param structure: Elements constituting the ITD
+        :param causative_event: event causing the fusion
+        :param assay: how knowledge of the fusion was obtained
+        :param regulatory_element: affected regulatory elements
+        :param critical_functional_domains: lost or preserved functional domains
+        :param reading_frame_preserved: ``True`` if reading frame is preserved.
+            ``False`` otherwise
+        :return: InternalTandemDuplication if construction successful
+        :raise: FUSORParametersException if given incorrect fusion properties
+        """
+        try:
+            return InternalTandemDuplication(
+                structure=structure,
+                regulatoryElement=regulatory_element,
+                causativeEvent=causative_event,
+                assay=assay,
+                readingFramePreserved=reading_frame_preserved,
+                criticalFunctionalDomains=critical_functional_domains,
+            )
+        except ValidationError as e:
+            error_message = get_error_message(e)
+            raise FUSORParametersException(error_message) from e
+
     async def transcript_segment_element(
         self,
         tx_to_genomic_coords: bool = True,
@@ -423,6 +455,7 @@ class FUSOR:
         start: int,
         end: int,
         use_minimal_gene: bool = True,
+        coordinate_type: CoordinateType = CoordinateType.RESIDUE,
         seq_id_target_namespace: str | None = None,
     ) -> tuple[FunctionalDomain | None, str | None]:
         """Build FunctionalDomain object.
@@ -436,6 +469,9 @@ class FUSOR:
         :param end: end position on sequence
         :param use_minimal_gene: ``True`` if minimal gene object (``id``, ``label``) will be used. ``False`` if gene-normalizer's gene
             object will be used
+        :param coordinate_type: The coordinate type that is being supplied
+            for ``start`` and ``end``. This is set to residue coordinates
+            by default
         :param seq_id_target_namespace: If want to use digest for ``sequence_id``, set
             this to the namespace you want the digest for. Otherwise, leave as ``None``.
         :return: Tuple with FunctionalDomain and None value for warnings if
@@ -450,7 +486,7 @@ class FUSOR:
             return None, msg
 
         seq, warning = self.cool_seq_tool.seqrepo_access.get_reference_sequence(
-            sequence_id, start, end
+            sequence_id, start, end, coordinate_type=coordinate_type
         )
 
         if not seq:
@@ -463,7 +499,10 @@ class FUSOR:
             return None, warning
 
         loc_descr = self._sequence_location(
-            start, end, sequence_id, seq_id_target_namespace=seq_id_target_namespace
+            start - 1 if coordinate_type == CoordinateType.RESIDUE else start,
+            end,
+            sequence_id,
+            seq_id_target_namespace=seq_id_target_namespace,
         )
 
         try:
@@ -486,15 +525,32 @@ class FUSOR:
         self,
         regulatory_class: RegulatoryClass,
         gene: str,
+        feature_id: str | None = None,
+        sequence_id: str | None = None,
+        start: int | None = None,
+        end: int | None = None,
+        seq_id_target_namespace: str | None = None,
+        coordinate_type: CoordinateType = CoordinateType.RESIDUE,
         use_minimal_gene: bool = True,
     ) -> tuple[RegulatoryElement | None, str | None]:
         """Create RegulatoryElement
 
         :param regulatory_class: one of {"promoter", "enhancer"}
         :param gene: gene term to fetch normalized gene object for
+        :param feature_id: The feature ID for the regulatory element
+        :param sequence_id: Genomic sequence on which provided coordinates exist
+        :param start: Start position on sequence
+        :param end: Etart position on sequence
+        :param seq_id_target_namespace: If want to use digest for
+            ``sequence_id``, set this to the namespace you want the digest for.
+              Otherwise, leave as ``None``.
+        :param coordinate_type: The coordinate type that is being supplied
+            for ``start`` and ``end``. This is set to residue coordinates
+            by default
         :param use_minimal_gene: whether to use the minimal gene object
-        :return: Tuple with RegulatoryElement instance and None value for warnings if
-            successful, or a None value and warning message if unsuccessful
+        :return: Tuple with RegulatoryElement instance and None value for
+            warnings if successful, or a None value and warning message if
+            unsuccessful
         """
         gene_descr, warning = self._normalized_gene(
             gene, use_minimal_gene=use_minimal_gene
@@ -502,10 +558,42 @@ class FUSOR:
         if not gene_descr:
             return None, warning
 
+        if coordinate_type == CoordinateType.RESIDUE:
+            if start == 0:
+                return (
+                    None,
+                    "start must exceed 0 if using residue coordinates to construct the feature_location",
+                )
+            if end == 0:
+                return (
+                    None,
+                    "end must exceed 0 if using residue coordinates to construct the feature_location",
+                )
+
+        use_feat_location = any(loc_var for loc_var in (sequence_id, start, end))
+        if use_feat_location:
+            if not sequence_id or not start or not end:
+                return (
+                    None,
+                    "sequence_id, start, and end must all be provided to construct the feature_location",
+                )
+            feat_location = self._sequence_location(
+                start - 1 if coordinate_type == CoordinateType.RESIDUE else start,
+                end,
+                sequence_id,
+                seq_id_target_namespace=seq_id_target_namespace,
+            )
+            feat_location = GenomicLocation(
+                **feat_location.model_dump(exclude="name"), name=sequence_id
+            )
+
         try:
             return (
                 RegulatoryElement(
-                    regulatoryClass=regulatory_class, associatedGene=gene_descr
+                    regulatoryClass=regulatory_class,
+                    associatedGene=gene_descr,
+                    featureId=feature_id,
+                    featureLocation=feat_location if use_feat_location else None,
                 ),
                 None,
             )
@@ -533,7 +621,9 @@ class FUSOR:
             sequence_id, seq_id_target_namespace
         )
 
-        refget_accession = translate_identifier(self.seqrepo, sequence_id)
+        refget_accession = self.seqrepo.translate_identifier(
+            identifier=sequence_id, target_namespaces="ga4gh"
+        )[0]
 
         sequence_location = SequenceLocation(
             start=start,
@@ -615,15 +705,16 @@ class FUSOR:
 
         if seq_id_target_namespace:
             try:
-                seq_id = translate_identifier(
-                    self.seqrepo, sequence_id, target_namespace=seq_id_target_namespace
-                )
-            except IDTranslationException:
+                seq_id = self.seqrepo.translate_identifier(
+                    identifier=sequence_id, target_namespaces=seq_id_target_namespace
+                )[0]
+            except KeyError as e:
                 _logger.warning(
                     "Unable to translate %s using %s as the target namespace",
                     sequence_id,
                     seq_id_target_namespace,
                 )
+                raise IDTranslationException from e
             else:
                 sequence_id = seq_id
         return sequence_id
