@@ -7,9 +7,8 @@ from enum import Enum
 from pathlib import Path
 from typing import Annotated, Any, Literal
 
-from civicpy.civic import MolecularProfile
 from cool_seq_tool.schemas import Strand
-from ga4gh.core.models import MappableConcept
+from ga4gh.core.models import Extension, MappableConcept
 from ga4gh.vrs.models import (
     LiteralSequenceExpression,
     SequenceLocation,
@@ -23,6 +22,7 @@ from pydantic import (
     StrictInt,
     StrictStr,
     StringConstraints,
+    field_validator,
     model_validator,
 )
 
@@ -56,9 +56,45 @@ class FUSORTypes(str, Enum):
     REGULATORY_ELEMENT = "RegulatoryElement"
     CATEGORICAL_FUSION = "CategoricalFusion"
     ASSAYED_FUSION = "AssayedFusion"
+    INTERNAL_TANDEM_DUPLICATION = "InternalTandemDuplication"
     CAUSATIVE_EVENT = "CausativeEvent"
 
 
+class GenomicLocation(SequenceLocation):
+    """Define GenomicLocation class"""
+
+    name: str
+
+    @field_validator("name")
+    def validate_genomic_location(cls, value: str):
+        """Validate that featureLocation only describes genomic coordinates
+        if provided
+
+        :param value: The value for `name`
+        :raises ValueError: If a non-chromosomal accession are provided to
+            `name`
+        """
+        if not value.startswith("NC_"):
+            msg = "`name` must be a RefSeq chromosomal accession that starts with `NC_`"
+            raise ValueError(msg)
+        return value
+
+    model_config = ConfigDict(
+        json_schema_extra={
+            "example": {
+                "id": "ga4gh:SL.9hqdPDfXC-m_t_bDH75FZHfaM6OKDtRw",
+                "name": "NC_000001.11",
+                "type": "SequenceLocation",
+                "sequenceReference": {
+                    "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
+                },
+                "start": 155593,
+                "end": 155610,
+            }
+        },
+    )
+
+
 class AdditionalFields(str, Enum):
     """Define possible fields that can be added to Fusion object."""
 
@@ -256,7 +292,7 @@ class TranscriptSegmentElement(BaseStructuralElement):
     coverage: BreakpointCoverage | None = None
     anchoredReads: AnchoredReads | None = None
 
-    @model_validator(mode="before")
+    @model_validator(mode="after")
     def check_exons(cls, values):
         """Check that at least one of {``exonStart``, ``exonEnd``} is set.
         If set, check that the corresponding ``elementGenomic`` field is set.
@@ -264,24 +300,24 @@ class TranscriptSegmentElement(BaseStructuralElement):
 
         """
         msg = "Must give values for either `exonStart`, `exonEnd`, or both"
-        exon_start = values.get("exonStart")
-        exon_end = values.get("exonEnd")
+        exon_start = values.exonStart
+        exon_end = values.exonEnd
         if (exon_start is None) and (exon_end is None):
             raise ValueError(msg)
 
         if exon_start:
-            if not values.get("elementGenomicStart"):
+            if not values.elementGenomicStart:
                 msg = "Must give `elementGenomicStart` if `exonStart` is given"
                 raise ValueError(msg)
         else:
-            values["exonStartOffset"] = None
+            values.exonStartOffset = None
 
         if exon_end:
-            if not values.get("elementGenomicEnd"):
+            if not values.elementGenomicEnd:
                 msg = "Must give `elementGenomicEnd` if `exonEnd` is given"
                 raise ValueError(msg)
         else:
-            values["exonEndOffset"] = None
+            values.exonEndOffset = None
         return values
 
     model_config = ConfigDict(
@@ -313,6 +349,7 @@ class TranscriptSegmentElement(BaseStructuralElement):
                         "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
                     },
                     "end": 154192135,
+                    "extensions": [{"name": "is_exonic", "value": True}],
                 },
                 "elementGenomicEnd": {
                     "id": "ga4gh:SL.Lnne0bSsgjzmNkKsNnXg98FeJSrDJuLb",
@@ -324,6 +361,7 @@ class TranscriptSegmentElement(BaseStructuralElement):
                         "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
                     },
                     "start": 154170399,
+                    "extensions": [{"name": "is_exonic", "value": True}],
                 },
                 "coverage": {
                     "type": "BreakpointCoverage",
@@ -498,16 +536,16 @@ class RegulatoryElement(BaseModel):
     regulatoryClass: RegulatoryClass
     featureId: str | None = None
     associatedGene: MappableConcept | None = None
-    featureLocation: SequenceLocation | None = None
+    featureLocation: GenomicLocation | None = None
 
-    @model_validator(mode="before")
+    @model_validator(mode="after")
     def ensure_min_values(cls, values):
         """Ensure that one of {`featureId`, `featureLocation`}, and/or
         `associatedGene` is set.
         """
-        if not (
-            bool(values.get("featureId")) ^ bool(values.get("featureLocation"))
-        ) and not (values.get("associatedGene")):
+        if not (bool(values.featureId) ^ bool(values.featureLocation)) and not (
+            values.associatedGene
+        ):
             msg = (
                 "Must set 1 of {`featureId`, `associatedGene`} and/or `featureLocation`"
             )
@@ -521,6 +559,7 @@ class RegulatoryElement(BaseModel):
                 "regulatoryClass": "promoter",
                 "featureLocation": {
                     "id": "ga4gh:SL.9hqdPDfXC-m_t_bDH75FZHfaM6OKDtRw",
+                    "name": "NC_000001.11",
                     "type": "SequenceLocation",
                     "sequenceReference": {
                         "id": "refseq:NC_000001.11",
@@ -546,14 +585,25 @@ class FusionType(str, Enum):
         return {c.value for c in cls}
 
 
-class AbstractFusion(BaseModel, ABC):
-    """Define AbstractFusion class"""
+class DuplicationType(str, Enum):
+    """Define possible Duplication types"""
+
+    INTERNAL_TANDEM_DUPLICATION = FUSORTypes.INTERNAL_TANDEM_DUPLICATION.value
+
+    @classmethod
+    def values(cls) -> set[str]:
+        """Provide all possible enum values."""
+        return {c.value for c in cls}
+
+
+class AbstractTranscriptStructuralVariant(BaseModel, ABC):
+    """Define AbstractTranscriptStructuralVariant class"""
 
-    type: FusionType
     regulatoryElement: RegulatoryElement | None = None
     structure: list[BaseStructuralElement]
+    fivePrimeJunction: StrictStr | None = None
+    threePrimeJunction: StrictStr | None = None
     readingFramePreserved: StrictBool | None = None
-    viccNomenclature: StrictStr | None = None
 
     @classmethod
     def _access_object_attr(
@@ -615,12 +665,29 @@ class AbstractFusion(BaseModel, ABC):
     @model_validator(mode="before")
     def enforce_abc(cls, values):
         """Ensure only subclasses can be instantiated."""
-        if cls.__name__ == "AbstractFusion":
-            msg = "Cannot instantiate Fusion abstract class"
-            raise ValueError(msg)
+        if cls is AbstractTranscriptStructuralVariant:
+            msg = (
+                "Cannot instantiate AbstractTranscriptStructuralVariant abstract class"
+            )
+            raise TypeError(msg)
         return values
 
+
+class AbstractFusion(AbstractTranscriptStructuralVariant):
+    """Define AbstractFusion class"""
+
+    type: FusionType
+    viccNomenclature: StrictStr | None = None
+
     @model_validator(mode="before")
+    def enforce_abc(cls, values):
+        """Ensure only subclasses can be instantiated."""
+        if cls is AbstractFusion:
+            msg = "Cannot instantiate Fusion abstract class"
+            raise TypeError(msg)
+        return values
+
+    @model_validator(mode="after")
     def enforce_element_quantities(cls, values):
         """Ensure minimum # of elements, and require > 1 unique genes.
 
@@ -630,14 +697,14 @@ class AbstractFusion(BaseModel, ABC):
         an unknown partner), then we raise an error.
         """
         qt_error_msg = (
-            "Fusions must contain >= 2 structural elements, or >=1 structural element "
+            "Fusions must contain >= 2 structural elements, or >= 1 structural element "
             "and a regulatory element"
         )
-        structure = values.get("structure", [])
+        structure = values.structure
         if not structure:
             raise ValueError(qt_error_msg)
         num_structure = len(structure)
-        reg_element = values.get("regulatoryElement")
+        reg_element = values.regulatoryElement
         if (num_structure + bool(reg_element)) < 2:
             raise ValueError(qt_error_msg)
 
@@ -852,8 +919,7 @@ class CategoricalFusion(AbstractFusion):
     type: Literal[FUSORTypes.CATEGORICAL_FUSION] = FUSORTypes.CATEGORICAL_FUSION
     criticalFunctionalDomains: list[FunctionalDomain] | None = None
     structure: list[CategoricalFusionElement]
-    civicMolecularProfiles: list[MolecularProfile] | None = None
-    moaAssertion: dict | None = None
+    extensions: list[Extension] | None = None
 
     model_config = ConfigDict(
         arbitrary_types_allowed=True,
@@ -906,6 +972,7 @@ class CategoricalFusion(AbstractFusion):
                                 "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
                             },
                             "end": 154192135,
+                            "extensions": [{"name": "is_exonic", "value": True}],
                         },
                         "elementGenomicEnd": {
                             "id": "ga4gh:SL.Lnne0bSsgjzmNkKsNnXg98FeJSrDJuLb",
@@ -917,6 +984,7 @@ class CategoricalFusion(AbstractFusion):
                                 "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
                             },
                             "start": 154170399,
+                            "extensions": [{"name": "is_exonic", "value": True}],
                         },
                     },
                     {
@@ -953,14 +1021,174 @@ class CategoricalFusion(AbstractFusion):
 Fusion = CategoricalFusion | AssayedFusion
 
 
+InternalTandemDuplicationElements = Annotated[
+    TranscriptSegmentElement
+    | GeneElement
+    | TemplatedSequenceElement
+    | LinkerElement
+    | UnknownGeneElement
+    | MultiplePossibleGenesElement,
+    Field(discriminator="type"),
+]
+
+
+class InternalTandemDuplication(AbstractTranscriptStructuralVariant):
+    """Internal tandem duplications are repeated transcribed elements within a gene as
+    a result of focal duplications. These can be described in both an assayed and
+    categorical context. These events differ from fusions in that the same gene symbol
+    must be used for both event partners, indicating a duplication.
+    """
+
+    type: Literal[FUSORTypes.INTERNAL_TANDEM_DUPLICATION] = (
+        FUSORTypes.INTERNAL_TANDEM_DUPLICATION
+    )
+    structure: list[InternalTandemDuplicationElements]
+    causativeEvent: CausativeEvent | None = None
+    assay: Assay | None = None
+    contig: ContigSequence | None = None
+    readData: ReadData | None = None
+    criticalFunctionalDomains: list[FunctionalDomain] | None = None
+
+    @model_validator(mode="after")
+    def enforce_itd_element_quantities(cls, values):
+        """Ensure minimum # of elements for InternalTandemDuplications (ITDs)
+
+        To validate the unique genes rule, we extract gene IDs from the elements that
+        designate genes, and take the number of total elements. If there is only one
+        unique gene ID, and there are no non-gene-defining elements (such as
+        an unknown partner), then we raise an error.
+        """
+        qt_error_msg = (
+            "ITDs must contain >= 2 structural elements, or >= 1 structural element "
+            "and a regulatory element"
+        )
+        structure = values.structure
+        if not structure:
+            raise ValueError(qt_error_msg)
+        num_structure = len(structure)
+        reg_element = values.regulatoryElement
+        if (num_structure + bool(reg_element)) < 2:
+            raise ValueError(qt_error_msg)
+
+        uq_gene_msg = "ITDs must be formed from only one unique gene."
+        gene_ids = []
+
+        for element in structure:
+            gene_id = cls._fetch_gene_id_or_name(obj=element)
+            if gene_id:
+                gene_ids.append(gene_id)
+
+        unique_gene_ids = set(gene_ids)
+        if len(unique_gene_ids) != 1:
+            raise ValueError(uq_gene_msg)
+        return values
+
+    # Provided example is a duplication event of exons 1-8 of TPM3
+    model_config = ConfigDict(
+        arbitrary_types_allowed=True,
+        json_schema_extra={
+            "example": {
+                "type": "InternalTandemDuplication",
+                "readingFramePreserved": True,
+                "structure": [
+                    {
+                        "type": "TranscriptSegmentElement",
+                        "transcript": "refseq:NM_152263.3",
+                        "strand": -1,
+                        "exonStart": 1,
+                        "exonStartOffset": 0,
+                        "exonEnd": 8,
+                        "exonEndOffset": 0,
+                        "gene": {
+                            "primaryCoding": {
+                                "id": "hgnc:12012",
+                                "code": "HGNC:12012",
+                                "system": "https://www.genenames.org/data/gene-symbol-report/#!/hgnc_id/",
+                            },
+                            "conceptType": "Gene",
+                            "name": "TPM3",
+                        },
+                        "elementGenomicStart": {
+                            "id": "ga4gh:SL.Q8vkGp7_xR9vI0PQ7g1IvUUeQ4JlJG8l",
+                            "digest": "Q8vkGp7_xR9vI0PQ7g1IvUUeQ4JlJG8l",
+                            "type": "SequenceLocation",
+                            "sequenceReference": {
+                                "id": "refseq:NC_000001.11",
+                                "type": "SequenceReference",
+                                "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
+                            },
+                            "end": 154192135,
+                            "extensions": [{"name": "is_exonic", "value": True}],
+                        },
+                        "elementGenomicEnd": {
+                            "id": "ga4gh:SL.Lnne0bSsgjzmNkKsNnXg98FeJSrDJuLb",
+                            "digest": "Lnne0bSsgjzmNkKsNnXg98FeJSrDJuLb",
+                            "type": "SequenceLocation",
+                            "sequenceReference": {
+                                "id": "refseq:NC_000001.11",
+                                "type": "SequenceReference",
+                                "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
+                            },
+                            "start": 154170399,
+                            "extensions": [{"name": "is_exonic", "value": True}],
+                        },
+                    },
+                    {
+                        "type": "TranscriptSegmentElement",
+                        "transcript": "refseq:NM_152263.3",
+                        "strand": -1,
+                        "exonStart": 1,
+                        "exonStartOffset": 0,
+                        "exonEnd": 8,
+                        "exonEndOffset": 0,
+                        "gene": {
+                            "primaryCoding": {
+                                "id": "hgnc:12012",
+                                "code": "HGNC:12012",
+                                "system": "https://www.genenames.org/data/gene-symbol-report/#!/hgnc_id/",
+                            },
+                            "conceptType": "Gene",
+                            "name": "TPM3",
+                        },
+                        "elementGenomicStart": {
+                            "id": "ga4gh:SL.Q8vkGp7_xR9vI0PQ7g1IvUUeQ4JlJG8l",
+                            "digest": "Q8vkGp7_xR9vI0PQ7g1IvUUeQ4JlJG8l",
+                            "type": "SequenceLocation",
+                            "sequenceReference": {
+                                "id": "refseq:NC_000001.11",
+                                "type": "SequenceReference",
+                                "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
+                            },
+                            "end": 154192135,
+                            "extensions": [{"name": "is_exonic", "value": True}],
+                        },
+                        "elementGenomicEnd": {
+                            "id": "ga4gh:SL.Lnne0bSsgjzmNkKsNnXg98FeJSrDJuLb",
+                            "digest": "Lnne0bSsgjzmNkKsNnXg98FeJSrDJuLb",
+                            "type": "SequenceLocation",
+                            "sequenceReference": {
+                                "id": "refseq:NC_000001.11",
+                                "type": "SequenceReference",
+                                "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
+                            },
+                            "start": 154170399,
+                            "extensions": [{"name": "is_exonic", "value": True}],
+                        },
+                    },
+                ],
+            }
+        },
+    )
+
+
 def save_fusions_cache(
-    fusions_list: list[AssayedFusion | CategoricalFusion],
+    variants_list: list[AssayedFusion | CategoricalFusion | InternalTandemDuplication],
     cache_name: str,
     cache_dir: Path | None = None,
 ) -> None:
     """Save a list of translated fusions as a cache
 
-    :param fusions_list: A list of FUSOR-translated fusions
+    :param variants_list: A list of FUSOR-translated fusions or ITDs
     :param cache_name: The name for the resultant cached file
     :param cache_dir: The location to store the cached file. If this parameter is
         not supplied, it will default to storing data in the `FUSOR_DATA_DIR`
@@ -973,4 +1201,4 @@ def save_fusions_cache(
     if output_file.exists():
         _logger.warning("Cached fusions file already exists. Overwriting with new file")
     with output_file.open("wb") as f:
-        pickle.dump(fusions_list, f)
+        pickle.dump(variants_list, f)
