@@ -7,10 +7,9 @@ import re
 from abc import ABC, abstractmethod
 from enum import Enum
 
-import polars as pl
 from civicpy.civic import ExonCoordinate, MolecularProfile
 from cool_seq_tool.schemas import Assembly, CoordinateType
-from ga4gh.core.models import MappableConcept
+from ga4gh.core.models import Extension, MappableConcept
 from ga4gh.vrs.models import LiteralSequenceExpression
 from pydantic import BaseModel
 
@@ -38,6 +37,7 @@ from fusor.models import (
     ContigSequence,
     EventType,
     GeneElement,
+    InternalTandemDuplication,
     LinkerElement,
     MultiplePossibleGenesElement,
     ReadData,
@@ -46,6 +46,7 @@ from fusor.models import (
     TranscriptSegmentElement,
     UnknownGeneElement,
 )
+from fusor.nomenclature import gene_nomenclature, tx_segment_nomenclature
 
 _logger = logging.getLogger(__name__)
 
@@ -71,9 +72,9 @@ class Translator(ABC):
         """
         self.fusor = fusor
 
-    def _format_fusion(
+    def _format_fusion_itd(
         self,
-        fusion_type: AssayedFusion | CategoricalFusion,
+        variant_type: AssayedFusion | CategoricalFusion | InternalTandemDuplication,
         gene_5prime: GeneElement | UnknownGeneElement | MultiplePossibleGenesElement,
         gene_3prime: GeneElement | UnknownGeneElement | MultiplePossibleGenesElement,
         tr_5prime: TranscriptSegmentElement | None = None,
@@ -86,10 +87,11 @@ class Translator(ABC):
         reads: ReadData | None = None,
         molecular_profiles: list[MolecularProfile] | None = None,
         moa_assertion: dict | None = None,
-    ) -> AssayedFusion | CategoricalFusion:
-        """Format classes to create AssayedFusion objects
+    ) -> AssayedFusion | CategoricalFusion | InternalTandemDuplication:
+        """Format classes to create Fusion and Internal Tandem Duplication (ITD) objects
 
-        :param fusion_type: If the fusion is an AssayedFusion or CategoricalFusion
+        :param variant_type: If the fusion is an AssayedFusion, CategoricalFusion, or
+            InternalTandemDuplication
         :param gene_5prime: 5'prime GeneElement or UnknownGeneElement or MultiplePossibleGenesElement
         :param gene_3prime: 3'prime GeneElement or UnknownGeneElement or MultiplePossibleGenesElement
         :param tr_5prime: 5'prime TranscriptSegmentElement
@@ -110,8 +112,6 @@ class Translator(ABC):
             "assay": assay,
             "contig": contig,
             "readData": reads,
-            "civicMolecularProfiles": molecular_profiles,
-            "moaAssertion": moa_assertion,
         }
         if not tr_5prime and not tr_3prime:
             params["structure"] = [gene_5prime, gene_3prime]
@@ -123,11 +123,33 @@ class Translator(ABC):
             params["structure"] = [tr_5prime, tr_3prime]
         if linker_sequence:
             params["structure"].insert(1, linker_sequence)
-        fusion = fusion_type(**params)
+
+        extensions = [
+            Extension(name=name, value=value)
+            for name, value in [
+                ("civicMolecularProfiles", molecular_profiles),
+                ("moaAssertion", moa_assertion),
+            ]
+            if value
+        ]
+        params["extensions"] = extensions
+        variant = variant_type(**params)
 
         # Assign VICC Nomenclature string to fusion event
-        fusion.viccNomenclature = self.fusor.generate_nomenclature(fusion)
-        return fusion
+        if not isinstance(variant, InternalTandemDuplication):
+            variant.viccNomenclature = self.fusor.generate_nomenclature(variant)
+        else:
+            variant.fivePrimeJunction = (
+                tx_segment_nomenclature(tr_5prime)
+                if tr_5prime
+                else gene_nomenclature(gene_5prime)
+            )
+            variant.threePrimeJunction = (
+                tx_segment_nomenclature(tr_3prime)
+                if tr_3prime
+                else gene_nomenclature(gene_3prime)
+            )
+        return variant
 
     def _get_causative_event(
         self, chrom1: str, chrom2: str, descr: str | None = None
@@ -192,12 +214,7 @@ class Translator(ABC):
         :param gene_3prime: The 3' gene partner or UnknownGeneElement or MultiplePossibleGenesElement
         :return ``True`` if the gene symbols are different, ``False`` if not
         """
-        if gene_5prime != gene_3prime:
-            return True
-        _logger.error(
-            "The supplied fusion is not valid as the two fusion partners are the same"
-        )
-        return False
+        return gene_5prime != gene_3prime
 
     def _get_genomic_ac(self, chrom: str, build: Assembly) -> str:
         """Return a RefSeq genomic accession given a chromosome and a reference build
@@ -209,12 +226,12 @@ class Translator(ABC):
         """
         sr = self.fusor.cool_seq_tool.seqrepo_access
         alias_list, errors = sr.translate_identifier(
-            f"{build}:{chrom}", target_namespaces="refseq"
+            f"{build.value}:{chrom}", target_namespaces="refseq"
         )
         if errors:
             statement = f"Genomic accession for {chrom} could not be retrieved"
             _logger.error(statement)
-            raise ValueError
+            raise ValueError(statement)
         return alias_list[0].split(":")[1]
 
     def _assess_gene_symbol(
@@ -277,7 +294,7 @@ class Translator(ABC):
     @abstractmethod
     async def translate(
         self, fusion_data: BaseModel, coordinate_type: CoordinateType, rb: Assembly
-    ) -> AssayedFusion | CategoricalFusion:
+    ) -> AssayedFusion | CategoricalFusion | InternalTandemDuplication:
         """Define abstract translate method
 
         :param fusion_data: The fusion data from a fusion caller
@@ -308,10 +325,13 @@ class JAFFATranslator(Translator):
         genes = jaffa.fusion_genes.split(":")
         fusion_partners = self._process_gene_symbols(genes[0], genes[1], Caller.JAFFA)
 
-        if not self._are_fusion_partners_different(
-            fusion_partners.gene_5prime, fusion_partners.gene_3prime
-        ):
-            return None
+        variant_type = (
+            AssayedFusion
+            if self._are_fusion_partners_different(
+                fusion_partners.gene_5prime, fusion_partners.gene_3prime
+            )
+            else InternalTandemDuplication
+        )
 
         if not isinstance(fusion_partners.gene_5prime_element, UnknownGeneElement):
             tr_5prime = await self.fusor.transcript_segment_element(
@@ -328,7 +348,12 @@ class JAFFATranslator(Translator):
             tr_3prime = await self.fusor.transcript_segment_element(
                 tx_to_genomic_coords=False,
                 genomic_ac=self._get_genomic_ac(jaffa.chrom2, rb),
-                seg_start_genomic=jaffa.base2,
+                seg_start_genomic=jaffa.base2
+                if variant_type != InternalTandemDuplication
+                else None,
+                seg_end_genomic=jaffa.base2
+                if variant_type == InternalTandemDuplication
+                else None,
                 gene=fusion_partners.gene_3prime,
                 coordinate_type=coordinate_type,
                 starting_assembly=rb,
@@ -348,8 +373,8 @@ class JAFFATranslator(Translator):
             spanning=SpanningReads(spanningReads=jaffa.spanning_pairs),
         )
 
-        return self._format_fusion(
-            AssayedFusion,
+        return self._format_fusion_itd(
+            variant_type,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
             tr_5prime
@@ -384,10 +409,13 @@ class STARFusionTranslator(Translator):
         gene2 = star_fusion.right_gene.split("^")[0]
 
         fusion_partners = self._process_gene_symbols(gene1, gene2, Caller.STAR_FUSION)
-        if not self._are_fusion_partners_different(
-            fusion_partners.gene_5prime, fusion_partners.gene_3prime
-        ):
-            return None
+        variant_type = (
+            AssayedFusion
+            if self._are_fusion_partners_different(
+                fusion_partners.gene_5prime, fusion_partners.gene_3prime
+            )
+            else InternalTandemDuplication
+        )
 
         five_prime = star_fusion.left_breakpoint.split(":")
         three_prime = star_fusion.right_breakpoint.split(":")
@@ -405,7 +433,12 @@ class STARFusionTranslator(Translator):
         tr_3prime = await self.fusor.transcript_segment_element(
             tx_to_genomic_coords=False,
             genomic_ac=self._get_genomic_ac(three_prime[0], rb),
-            seg_start_genomic=int(three_prime[1]),
+            seg_start_genomic=int(three_prime[1])
+            if variant_type != InternalTandemDuplication
+            else None,
+            seg_end_genomic=int(three_prime[1])
+            if variant_type == InternalTandemDuplication
+            else None,
             gene=fusion_partners.gene_3prime,
             coordinate_type=coordinate_type,
             starting_assembly=rb,
@@ -420,8 +453,8 @@ class STARFusionTranslator(Translator):
             spanning=SpanningReads(spanningReads=star_fusion.spanning_frag_count),
         )
 
-        return self._format_fusion(
-            AssayedFusion,
+        return self._format_fusion_itd(
+            variant_type,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
             tr_5prime
@@ -456,10 +489,13 @@ class FusionCatcherTranslator(Translator):
             fusion_catcher.three_prime_partner,
             Caller.FUSION_CATCHER,
         )
-        if not self._are_fusion_partners_different(
-            fusion_partners.gene_5prime, fusion_partners.gene_3prime
-        ):
-            return None
+        variant_type = (
+            AssayedFusion
+            if self._are_fusion_partners_different(
+                fusion_partners.gene_5prime, fusion_partners.gene_3prime
+            )
+            else InternalTandemDuplication
+        )
 
         five_prime = fusion_catcher.five_prime_fusion_point.split(":")
         three_prime = fusion_catcher.three_prime_fusion_point.split(":")
@@ -477,7 +513,12 @@ class FusionCatcherTranslator(Translator):
         tr_3prime = await self.fusor.transcript_segment_element(
             tx_to_genomic_coords=False,
             genomic_ac=self._get_genomic_ac(three_prime[0], rb),
-            seg_start_genomic=int(three_prime[1]),
+            seg_start_genomic=int(three_prime[1])
+            if variant_type != InternalTandemDuplication
+            else None,
+            seg_end_genomic=int(three_prime[1])
+            if variant_type == InternalTandemDuplication
+            else None,
             gene=fusion_partners.gene_3prime,
             coordinate_type=coordinate_type,
             starting_assembly=rb,
@@ -493,8 +534,8 @@ class FusionCatcherTranslator(Translator):
         )
         contig = ContigSequence(contig=fusion_catcher.fusion_sequence)
 
-        return self._format_fusion(
-            AssayedFusion,
+        return self._format_fusion_itd(
+            variant_type,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
             tr_5prime
@@ -509,70 +550,6 @@ class FusionCatcherTranslator(Translator):
         )
 
 
-class FusionMapTranslator(Translator):
-    """Initialize FusionMapTranslator class"""
-
-    async def translate(
-        self, fmap_row: pl.DataFrame, coordinate_type: CoordinateType, rb: Assembly
-    ) -> AssayedFusion:
-        """Parse FusionMap output to create FUSOR AssayedFusion object
-
-        :param fmap_row: A row of FusionMap output
-        :param rb: The reference build used to call the fusion
-        :param coordinate_type: If the coordinate is inter-residue or residue
-        :return: An AssayedFusion object, if construction is successful
-        """
-        gene1 = fmap_row.get_column("KnownGene1").item()
-        gene2 = fmap_row.get_column("KnownGene2").item()
-        gene_5prime = self._get_gene_element(gene1, "fusion_map").gene.name
-        gene_3prime = self._get_gene_element(gene2, "fusion_map").gene.name
-
-        if not self._are_fusion_partners_different(gene_5prime, gene_3prime):
-            return None
-
-        tr_5prime = await self.fusor.transcript_segment_element(
-            tx_to_genomic_coords=False,
-            genomic_ac=self._get_genomic_ac(
-                fmap_row.get_column("Chromosome1").item(), rb
-            ),
-            seg_end_genomic=int(fmap_row.get_column("Position1").item()),
-            gene=gene_5prime,
-            coordinate_type=coordinate_type,
-            starting_assembly=rb,
-        )
-        tr_5prime = tr_5prime[0]
-
-        tr_3prime = await self.fusor.transcript_segment_element(
-            tx_to_genomic_coords=False,
-            genomic_ac=self._get_genomic_ac(
-                fmap_row.get_column("Chromosome2").item(), rb
-            ),
-            seg_start_genomic=int(fmap_row.get_column("Position2").item()),
-            gene=gene_3prime,
-            coordinate_type=coordinate_type,
-            starting_assembly=rb,
-        )
-        tr_3prime = tr_3prime[0]
-
-        # Combine columns to create fusion annotation string"
-        descr = (
-            fmap_row.get_column("FusionGene").item()
-            + ","
-            + fmap_row.get_column("SplicePatternClass").item()
-            + ","
-            + fmap_row.get_column("FrameShiftClass").item()
-        )
-        ce = self._get_causative_event(
-            fmap_row.get_column("Chromosome1").item(),
-            fmap_row.get_column("Chromosome2").item(),
-            descr,
-        )
-        rf = bool(fmap_row.get_column("FrameShiftClass").item() == "InFrame")
-        return self._format_fusion(
-            AssayedFusion, gene_5prime, gene_3prime, tr_5prime, tr_3prime, ce, rf
-        )
-
-
 class ArribaTranslator(Translator):
     """Initialize ArribaTranslator class"""
 
@@ -596,21 +573,33 @@ class ArribaTranslator(Translator):
             arriba.gene1, arriba.gene2, Caller.ARRIBA
         )
 
-        if not self._are_fusion_partners_different(
-            fusion_partners.gene_5prime, fusion_partners.gene_3prime
-        ):
-            return None
+        variant_type = (
+            AssayedFusion
+            if self._are_fusion_partners_different(
+                fusion_partners.gene_5prime, fusion_partners.gene_3prime
+            )
+            else InternalTandemDuplication
+        )
 
-        strand1 = arriba.strand1.split("/")[1]  # Determine strand that is transcribed
-        strand2 = arriba.strand2.split("/")[1]  # Determine strand that is transcribed
-        if strand1 == "+":
-            gene1_seg_start = arriba.direction1 == "upstream"
+        if variant_type is InternalTandemDuplication:
+            # Assume breakpoint describes segment end if the event is an ITD
+            gene1_seg_start = False
+            gene2_seg_start = False
         else:
-            gene1_seg_start = arriba.direction1 == "downstream"
-        if strand2 == "+":
-            gene2_seg_start = arriba.direction2 == "upstream"
-        else:
-            gene2_seg_start = arriba.direction2 == "downstream"
+            strand1 = arriba.strand1.split("/")[
+                1
+            ]  # Determine strand that is transcribed
+            strand2 = arriba.strand2.split("/")[
+                1
+            ]  # Determine strand that is transcribed
+            if strand1 == "+":
+                gene1_seg_start = arriba.direction1 == "upstream"
+            else:
+                gene1_seg_start = arriba.direction1 == "downstream"
+            if strand2 == "+":
+                gene2_seg_start = arriba.direction2 == "upstream"
+            else:
+                gene2_seg_start = arriba.direction2 == "downstream"
 
         breakpoint1 = arriba.breakpoint1.split(":")
         breakpoint2 = arriba.breakpoint2.split(":")
@@ -670,8 +659,8 @@ class ArribaTranslator(Translator):
             else None
         )
 
-        return self._format_fusion(
-            AssayedFusion,
+        return self._format_fusion_itd(
+            variant_type,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
             tr_5prime
@@ -703,6 +692,8 @@ class CiceroTranslator(Translator):
         :param coordinate_type: If the coordinate is inter-residue or residue
         :param rb: The reference build used to call the fusion
         :return: An AssayedFusion object, if construction is successful
+        :raises RuntimeError: If CICERO annotations indicate the fusion is not
+            valid
         """
         # Check if gene symbols have valid formatting. CICERO can output two or more
         # gene symbols for `gene_5prime` or `gene_3prime`, which are separated by a comma. As
@@ -710,8 +701,8 @@ class CiceroTranslator(Translator):
         # these events
         if "," in cicero.gene_5prime or "," in cicero.gene_3prime:
             msg = "Ambiguous gene symbols are reported by CICERO for at least one of the fusion partners"
-            _logger.warning(msg)
-            return msg
+            _logger.error(msg)
+            raise RuntimeError(msg)
 
         fusion_partners = self._process_gene_symbols(
             cicero.gene_5prime, cicero.gene_3prime, Caller.CICERO
@@ -721,13 +712,16 @@ class CiceroTranslator(Translator):
         # has biological meaning
         if cicero.sv_ort != ">":
             msg = "CICERO annotation indicates that this event does not have confident biological meaning"
-            _logger.warning(msg)
-            return msg
+            _logger.error(msg)
+            raise RuntimeError(msg)
 
-        if not self._are_fusion_partners_different(
-            fusion_partners.gene_5prime, fusion_partners.gene_3prime
-        ):
-            return None
+        variant_type = (
+            AssayedFusion
+            if self._are_fusion_partners_different(
+                fusion_partners.gene_5prime, fusion_partners.gene_3prime
+            )
+            else InternalTandemDuplication
+        )
 
         tr_5prime = await self.fusor.transcript_segment_element(
             tx_to_genomic_coords=False,
@@ -744,7 +738,12 @@ class CiceroTranslator(Translator):
         tr_3prime = await self.fusor.transcript_segment_element(
             tx_to_genomic_coords=False,
             genomic_ac=self._get_genomic_ac(cicero.chr_3prime, rb),
-            seg_start_genomic=cicero.pos_3prime,
+            seg_start_genomic=cicero.pos_3prime
+            if variant_type != InternalTandemDuplication
+            else None,
+            seg_end_genomic=cicero.pos_3prime
+            if variant_type == InternalTandemDuplication
+            else None,
             gene=fusion_partners.gene_3prime,
             coverage=BreakpointCoverage(fragmentCoverage=cicero.coverage_3prime),
             reads=AnchoredReads(reads=cicero.reads_3prime),
@@ -765,8 +764,8 @@ class CiceroTranslator(Translator):
             )
         contig = ContigSequence(contig=cicero.contig)
 
-        return self._format_fusion(
-            AssayedFusion,
+        return self._format_fusion_itd(
+            variant_type,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
             tr_5prime
@@ -780,57 +779,6 @@ class CiceroTranslator(Translator):
         )
 
 
-class MapSpliceTranslator(Translator):
-    """Initialize MapSpliceTranslator class"""
-
-    async def translate(
-        self, mapsplice_row: pl.DataFrame, coordinate_type: CoordinateType, rb: Assembly
-    ) -> AssayedFusion:
-        """Parse MapSplice output to create AssayedFusion object
-
-        :param mapsplice_row: A row of MapSplice output
-        :param rb: The reference build used to call the fusion
-        :param coordinate_type: If the coordinate is inter-residue or residue
-        :retun: An AssayedFusion object, if construction is successful
-        """
-        gene1 = mapsplice_row[60].strip(",")
-        gene2 = mapsplice_row[61].strip(",")
-        gene_5prime_element = self._get_gene_element(gene1, "mapsplice")
-        gene_3prime_element = self._get_gene_element(gene2, "mapsplice")
-        gene_5prime = gene_5prime_element.gene.name
-        gene_3prime = gene_3prime_element.gene.name
-
-        if not self._are_fusion_partners_different(gene_5prime, gene_3prime):
-            return None
-
-        tr_5prime = await self.fusor.transcript_segment_element(
-            tx_to_genomic_coords=False,
-            genomic_ac=self._get_genomic_ac(mapsplice_row[0].split("~")[0], rb),
-            seg_end_genomic=int(mapsplice_row[1]),
-            gene=gene_5prime,
-            coordinate_type=coordinate_type,
-            starting_assembly=rb,
-        )
-        tr_5prime = tr_5prime[0]
-
-        tr_3prime = await self.fusor.transcript_segment_element(
-            tx_to_genomic_coords=False,
-            genomic_ac=self._get_genomic_ac(mapsplice_row[0].split("~")[1], rb),
-            seg_start_genomic=int(mapsplice_row[2]),
-            gene=gene_3prime,
-            coordinate_type=coordinate_type,
-            starting_assembly=rb,
-        )
-        tr_3prime = tr_3prime[0]
-
-        ce = self._get_causative_event(
-            mapsplice_row[0].split("~")[0], mapsplice_row[0].split("~")[1]
-        )
-        return self._format_fusion(
-            AssayedFusion, gene_5prime, gene_3prime, tr_5prime, tr_3prime, ce
-        )
-
-
 class EnFusionTranslator(Translator):
     """Initialize EnFusionTranslator class"""
 
@@ -851,10 +799,13 @@ class EnFusionTranslator(Translator):
             enfusion.gene_5prime, enfusion.gene_3prime, Caller.ENFUSION
         )
 
-        if not self._are_fusion_partners_different(
-            fusion_partners.gene_5prime, fusion_partners.gene_3prime
-        ):
-            return None
+        variant_type = (
+            AssayedFusion
+            if self._are_fusion_partners_different(
+                fusion_partners.gene_5prime, fusion_partners.gene_3prime
+            )
+            else InternalTandemDuplication
+        )
 
         tr_5prime = await self.fusor.transcript_segment_element(
             tx_to_genomic_coords=False,
@@ -869,7 +820,12 @@ class EnFusionTranslator(Translator):
         tr_3prime = await self.fusor.transcript_segment_element(
             tx_to_genomic_coords=False,
             genomic_ac=self._get_genomic_ac(enfusion.chr_3prime, rb),
-            seg_start_genomic=enfusion.break_3prime,
+            seg_start_genomic=enfusion.break_3prime
+            if variant_type != InternalTandemDuplication
+            else None,
+            seg_end_genomic=enfusion.break_3prime
+            if variant_type == InternalTandemDuplication
+            else None,
             gene=fusion_partners.gene_3prime,
             coordinate_type=coordinate_type,
             starting_assembly=rb,
@@ -880,8 +836,8 @@ class EnFusionTranslator(Translator):
             enfusion.chr_5prime,
             enfusion.chr_3prime,
         )
-        return self._format_fusion(
-            AssayedFusion,
+        return self._format_fusion_itd(
+            variant_type,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
             tr_5prime
@@ -914,10 +870,13 @@ class GenieTranslator(Translator):
             genie.site1_hugo, genie.site2_hugo, Caller.GENIE
         )
 
-        if not self._are_fusion_partners_different(
-            fusion_partners.gene_5prime, fusion_partners.gene_3prime
-        ):
-            return None
+        variant_type = (
+            AssayedFusion
+            if self._are_fusion_partners_different(
+                fusion_partners.gene_5prime, fusion_partners.gene_3prime
+            )
+            else InternalTandemDuplication
+        )
 
         tr_5prime = await self.fusor.transcript_segment_element(
             tx_to_genomic_coords=False,
@@ -932,7 +891,12 @@ class GenieTranslator(Translator):
         tr_3prime = await self.fusor.transcript_segment_element(
             tx_to_genomic_coords=False,
             genomic_ac=self._get_genomic_ac(genie.site2_chrom, rb),
-            seg_start_genomic=genie.site2_pos,
+            seg_start_genomic=genie.site2_pos
+            if variant_type != InternalTandemDuplication
+            else None,
+            seg_end_genomic=genie.site2_pos
+            if variant_type == InternalTandemDuplication
+            else None,
             gene=fusion_partners.gene_3prime,
             coordinate_type=coordinate_type,
             starting_assembly=rb,
@@ -945,8 +909,8 @@ class GenieTranslator(Translator):
             genie.annot,
         )
         rf = bool(genie.reading_frame == "in frame")
-        return self._format_fusion(
-            AssayedFusion,
+        return self._format_fusion_itd(
+            variant_type,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
             tr_5prime
@@ -1013,17 +977,20 @@ class CIVICTranslator(Translator):
             and not (coord.start and coord.stop)
         )
 
-    async def translate(self, civic: CIVIC) -> CategoricalFusion | None:
+    async def translate(self, civic: CIVIC) -> CategoricalFusion:
         """Convert CIViC record to Categorical Fusion
 
         :param civic A CIVIC object
         :return A CategoricalFusion object, if construction is successful
+        :raises: ValueError if a transcript accession and exon number is
+            provided without corresponding genomic breakpoints
         """
         if not (
             self._valid_exon_coords(civic.five_prime_end_exon_coords)
             and self._valid_exon_coords(civic.three_prime_start_exon_coords)
         ):
-            return None
+            msg = "Translation cannot proceed as GRCh37 transcripts and exons lacks genomic breakpoints"
+            raise ValueError(msg)
         fusion_partners = civic.vicc_compliant_name
         if fusion_partners.startswith("v::"):
             gene_5prime = "v"
@@ -1046,20 +1013,15 @@ class CIVICTranslator(Translator):
         fusion_partners = self._process_gene_symbols(
             gene_5prime, gene_3prime, KnowledgebaseList.CIVIC
         )
-        if not self._are_fusion_partners_different(
-            fusion_partners.gene_5prime, fusion_partners.gene_3prime
-        ):
-            return None
-
         tr_5prime = None
         if (
             isinstance(civic.five_prime_end_exon_coords, ExonCoordinate)
             and civic.five_prime_end_exon_coords.chromosome
         ):  # Process for cases where exon data is available for 5' transcript
-            rb = (
-                Assembly.GRCH37.value
+            rb: Assembly = (
+                Assembly.GRCH37
                 if civic.five_prime_end_exon_coords.reference_build == "GRCH37"
-                else Assembly.GRCH38.value
+                else Assembly.GRCH38
             )
             tr_5prime = await self.fusor.transcript_segment_element(
                 tx_to_genomic_coords=False,
@@ -1080,10 +1042,10 @@ class CIVICTranslator(Translator):
             isinstance(civic.three_prime_start_exon_coords, ExonCoordinate)
             and civic.three_prime_start_exon_coords.chromosome
         ):  # Process for case where exon data is available for 3' transcript
-            rb = (
-                Assembly.GRCH37.value
+            rb: Assembly = (
+                Assembly.GRCH37
                 if civic.three_prime_start_exon_coords.reference_build == "GRCH37"
-                else Assembly.GRCH38.value
+                else Assembly.GRCH38
             )
             tr_3prime = await self.fusor.transcript_segment_element(
                 tx_to_genomic_coords=False,
@@ -1099,7 +1061,7 @@ class CIVICTranslator(Translator):
             )
             tr_3prime = tr_3prime[0]
 
-        return self._format_fusion(
+        return self._format_fusion_itd(
             CategoricalFusion,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
@@ -1112,7 +1074,7 @@ class CIVICTranslator(Translator):
 class MOATranslator(Translator):
     """Initialize MOATranslator"""
 
-    def translate(self, moa_assertion: dict) -> CategoricalFusion | None:
+    def translate(self, moa_assertion: dict) -> CategoricalFusion:
         """Convert a MOA assertion to a CategoricalFusion object
 
         :param moa_assertion: A dictionary representing a MOA assertion. To note, MOA fusions
@@ -1120,8 +1082,7 @@ class MOATranslator(Translator):
             where both partners are listed, as we cannot definitively determine for
             cases where one gene symbol is provided if it describes the 5' or 3'
             partner.
-        :return: A CategoricalFusion object or None if both gene partners are not
-            provided.
+        :return: A CategoricalFusion object
         """
         bm = None
         for biomarker in moa_assertion["proposition"]["biomarkers"]:
@@ -1136,7 +1097,7 @@ class MOATranslator(Translator):
         fusion_partners = self._process_gene_symbols(
             gene_5prime, gene_3prime, KnowledgebaseList.MOA
         )
-        return self._format_fusion(
+        return self._format_fusion_itd(
             CategoricalFusion,
             fusion_partners.gene_5prime_element,
             fusion_partners.gene_3prime_element,
