@@ -3,7 +3,7 @@
 import copy
 
 import pytest
-from cool_seq_tool.schemas import Strand
+from cool_seq_tool.schemas import CoordinateType, Strand
 from ga4gh.core.models import Coding, MappableConcept
 from ga4gh.vrs.models import SequenceLocation
 
@@ -13,6 +13,8 @@ from fusor.models import (
     CategoricalFusion,
     FunctionalDomain,
     GeneElement,
+    GenomicLocation,
+    InternalTandemDuplication,
     LinkerElement,
     MultiplePossibleGenesElement,
     RegulatoryClass,
@@ -37,6 +39,20 @@ def braf_gene_obj_min():
     )
 
 
+@pytest.fixture(scope="module")
+def tpm3_gene_obj_min():
+    """Create minimal gene object for TPM3"""
+    return MappableConcept(
+        primaryCoding=Coding(
+            id="hgnc:12012",
+            code="HGNC:12012",
+            system="https://www.genenames.org/data/gene-symbol-report/#!/hgnc_id/",
+        ),
+        name="TPM3",
+        conceptType="Gene",
+    )
+
+
 @pytest.fixture(scope="module")
 def braf_gene_obj(braf_gene):
     """Create gene object for braf"""
@@ -141,6 +157,38 @@ def regulatory_element(braf_gene_obj):
     return RegulatoryElement(**params)
 
 
+@pytest.fixture(scope="module")
+def genomic_location_feature_location():
+    """Create test genomic location for feature location. Adapted from models.py"""
+    params = {
+        "id": "ga4gh:SL.-xC3omZDIKZEuotbbHWQMTC8sS3nOxTb",
+        "name": "NC_000001.11",
+        "type": "SequenceLocation",
+        "sequenceReference": {
+            "id": "refseq:NC_000001.11",
+            "refgetAccession": "SQ.Ya6Rs7DHhDeg7YaOSg1EoNi3U_nQ9SvO",
+            "type": "SequenceReference",
+        },
+        "start": 15455,
+        "end": 15456,
+        "extensions": [{"name": "is_exonic", "value": True}],
+    }
+    return GenomicLocation(**params)
+
+
+@pytest.fixture(scope="module")
+def regulatory_element_full(tpm3_gene_obj_min, genomic_location_feature_location):
+    """Create full regulatory element test fixture"""
+    params = {
+        "type": "RegulatoryElement",
+        "regulatoryClass": "enhancer",
+        "associatedGene": tpm3_gene_obj_min,
+        "featureID": "EH12345",
+        "featureLocation": genomic_location_feature_location,
+    }
+    return RegulatoryElement(**params)
+
+
 @pytest.fixture(scope="module")
 def regulatory_element_min(braf_gene_obj_min):
     """Create regulatory element test fixture with minimal gene object."""
@@ -217,6 +265,7 @@ def transcript_segment_element():
                 "type": "SequenceReference",
             },
             "end": 154192135,
+            "extensions": [{"name": "is_exonic", "value": True}],
         },
         "elementGenomicEnd": {
             "id": "ga4gh:SL.Lnne0bSsgjzmNkKsNnXg98FeJSrDJuLb",
@@ -228,6 +277,7 @@ def transcript_segment_element():
                 "type": "SequenceReference",
             },
             "start": 154170399,
+            "extensions": [{"name": "is_exonic", "value": True}],
         },
     }
     return TranscriptSegmentElement(**params)
@@ -264,6 +314,7 @@ def mane_transcript_segment_element():
                 "type": "SequenceReference",
             },
             "start": 9575887,
+            "extensions": [{"name": "is_exonic", "value": True}],
         },
     }
     return TranscriptSegmentElement(**params)
@@ -430,7 +481,7 @@ def test_fusion(
         f = fusor_instance.fusion(
             fusion_type="CategoricalFusion", structure=[linker_element]
         )
-    msg = "Fusions must contain >= 2 structural elements, or >=1 structural element and a regulatory element"
+    msg = "Fusions must contain >= 2 structural elements, or >= 1 structural element and a regulatory element"
     assert msg in str(excinfo.value)
 
     expected = copy.deepcopy(transcript_segment_element)
@@ -463,6 +514,25 @@ def test_fusion(
     assert msg in str(excinfo.value)
 
 
+def test_itd(
+    fusor_instance,
+    transcript_segment_element,
+    functional_domain,
+):
+    """Test that ITD construction works successfully"""
+    itd = fusor_instance.internal_tandem_duplication(
+        structure=[transcript_segment_element, transcript_segment_element],
+        critical_functional_domains=[functional_domain],
+    )
+    assert isinstance(itd, InternalTandemDuplication)
+
+    itd = fusor_instance.internal_tandem_duplication(
+        structure=[transcript_segment_element, UnknownGeneElement()],
+        critical_functional_domains=[functional_domain],
+    )
+    assert isinstance(itd, InternalTandemDuplication)
+
+
 @pytest.mark.asyncio
 async def test_transcript_segment_element(
     fusor_instance, transcript_segment_element, mane_transcript_segment_element
@@ -729,7 +799,7 @@ def test_functional_domain(
         "interpro:IPR001245",
         "BRAF",
         "NP_004324.2",
-        458,
+        459,
         712,
         use_minimal_gene=False,
     )
@@ -741,7 +811,7 @@ def test_functional_domain(
         "interpro:IPR001245",
         "BRAF",
         "NP_004324.2",
-        458,
+        459,
         712,
         use_minimal_gene=True,
     )
@@ -757,6 +827,7 @@ def test_functional_domain(
         712,
         seq_id_target_namespace="ga4gh",
         use_minimal_gene=True,
+        coordinate_type=CoordinateType.INTER_RESIDUE,
     )
     compare_domains(cd, functional_domain_seq_id)
 
@@ -770,6 +841,7 @@ def test_functional_domain(
         712,
         seq_id_target_namespace="ga4gh",
         use_minimal_gene=True,
+        coordinate_type=CoordinateType.INTER_RESIDUE,
     )
     assert cd[0] is None
     assert "Input should be 'lost' or 'preserved'" in cd[1]
@@ -781,7 +853,7 @@ def test_functional_domain(
         "interpro:IPR001245",
         "BRAF",
         "NM_004333.4",
-        458,
+        459,
         712,
         seq_id_target_namespace="ga4gh",
         use_minimal_gene=True,
@@ -797,7 +869,7 @@ def test_functional_domain(
         "interpro:IPR001245",
         "BRAF",
         accession,
-        458,
+        459,
         712,
         seq_id_target_namespace="ga4gh",
         use_minimal_gene=True,
@@ -812,7 +884,7 @@ def test_functional_domain(
         "interpro:IPR001245",
         "BRAF",
         "NP_004324.2",
-        458,
+        459,
         712000,
         seq_id_target_namespace="ga4gh",
         use_minimal_gene=True,
@@ -823,7 +895,9 @@ def test_functional_domain(
     )
 
 
-def test_regulatory_element(fusor_instance, regulatory_element, regulatory_element_min):
+def test_regulatory_element(
+    fusor_instance, regulatory_element, regulatory_element_min, regulatory_element_full
+):
     """Test regulatory_element method."""
 
     def compare_re(actual, expected):
@@ -835,9 +909,78 @@ def test_regulatory_element(fusor_instance, regulatory_element, regulatory_eleme
         assert actual.keys() == expected.keys()
         assert actual["type"] == expected["type"]
         compare_gene_obj(actual["associatedGene"], expected["associatedGene"])
+        if actual.get("featureID"):
+            assert actual["featureID"] == expected["featureID"]
+        if actual.get("featureLocation"):
+            assert actual["featureLocation"]["id"] == expected["featureLocation"]["id"]
+            assert (
+                actual["featureLocation"]["start"]
+                == expected["featureLocation"]["start"]
+            )
+            assert (
+                actual["featureLocation"]["end"] == expected["featureLocation"]["end"]
+            )
 
-    re = fusor_instance.regulatory_element(RegulatoryClass.PROMOTER, "BRAF")
+    re = fusor_instance.regulatory_element(
+        regulatory_class=RegulatoryClass.PROMOTER,
+        gene="BRAF",
+    )
     compare_re(re, regulatory_element_min)
 
-    re = fusor_instance.regulatory_element(RegulatoryClass.PROMOTER, "BRAF", False)
+    re = fusor_instance.regulatory_element(
+        regulatory_class=RegulatoryClass.PROMOTER, gene="BRAF", use_minimal_gene=False
+    )
     compare_re(re, regulatory_element)
+
+    re = fusor_instance.regulatory_element(
+        regulatory_class=RegulatoryClass.ENHANCER,
+        gene="TPM3",
+        feature_id="EH12345",
+        sequence_id="NC_000001.11",
+        start=15455,
+        end=15456,
+        coordinate_type=CoordinateType.INTER_RESIDUE,
+    )
+    compare_re(re, regulatory_element_full)
+
+    re = fusor_instance.regulatory_element(
+        regulatory_class=RegulatoryClass.ENHANCER,
+        gene="TPM3",
+        feature_id="EH12345",
+        sequence_id="NC_000001.11",
+        start=15455,
+        coordinate_type=CoordinateType.INTER_RESIDUE,
+    )
+    assert re[0] is None
+    assert (
+        re[1]
+        == "sequence_id, start, and end must all be provided to construct the feature_location"
+    )
+
+    re = fusor_instance.regulatory_element(
+        regulatory_class=RegulatoryClass.ENHANCER,
+        gene="TPM3",
+        feature_id="EH12345",
+        sequence_id="NC_000001.11",
+        start=0,
+        coordinate_type=CoordinateType.RESIDUE,
+    )
+    assert re[0] is None
+    assert (
+        re[1]
+        == "start must exceed 0 if using residue coordinates to construct the feature_location"
+    )
+
+    re = fusor_instance.regulatory_element(
+        regulatory_class=RegulatoryClass.ENHANCER,
+        gene="TPM3",
+        feature_id="EH12345",
+        sequence_id="NC_000001.11",
+        end=0,
+        coordinate_type=CoordinateType.RESIDUE,
+    )
+    assert re[0] is None
+    assert (
+        re[1]
+        == "end must exceed 0 if using residue coordinates to construct the feature_location"
+    )
