@@ -9,6 +9,7 @@ from pydantic import ValidationError
 from fusor.config import config
 from fusor.models import (
     AbstractFusion,
+    AbstractTranscriptStructuralVariant,
     AnchoredReads,
     Assay,
     AssayedFusion,
@@ -19,6 +20,7 @@ from fusor.models import (
     EventType,
     FunctionalDomain,
     GeneElement,
+    InternalTandemDuplication,
     LinkerElement,
     MultiplePossibleGenesElement,
     ReadData,
@@ -91,7 +93,7 @@ def gene_examples():
                 "code": "HGNC:1097",
                 "system": "https://www.genenames.org/data/gene-symbol-report/#!/hgnc_id/",
             },
-            "type": "Gene",
+            "conceptType": "Gene",
             "name": "BRAF",
         },
     ]
@@ -111,6 +113,7 @@ def sequence_locations():
             },
             "start": 15455,
             "end": 15456,
+            "extensions": [{"name": "is_exonic", "value": True}],
         },
         {
             "id": "ga4gh:SL.-xC3omZDIKZEuotbbHWQMTC8sS3nOxTb",
@@ -122,6 +125,7 @@ def sequence_locations():
             },
             "start": 15565,
             "end": 15566,
+            "extensions": [{"name": "is_exonic", "value": True}],
         },
         {
             "id": "ga4gh:SL.PPQ-aYd6dsSj7ulUEeqK8xZJP-yPrfdP",
@@ -133,6 +137,7 @@ def sequence_locations():
             },
             "start": 1,
             "end": 2,
+            "extensions": [{"name": "is_exonic", "value": True}],
         },
         {
             "id": "ga4gh:SL.OBeSv2B0pURlocL7viFiRwajew_GYGqN",
@@ -144,6 +149,7 @@ def sequence_locations():
             },
             "start": 2,
             "end": 3,
+            "extensions": [{"name": "is_exonic", "value": True}],
         },
         {
             "id": "ga4gh:SL.OBeSv2B0pURlocL7viFiRwajew_GYGqN",
@@ -170,6 +176,7 @@ def sequence_locations():
         {
             "id": "ga4gh:SL.VJLxl42yYoa-0ZMa8dfakhZfcP0nWgpl",
             "type": "SequenceLocation",
+            "name": "NP_001123617.1",
             "sequenceReference": {
                 "id": "refseq:NP_001123617.1",
                 "refgetAccession": "SQ.sv5egNzqN5koJQH6w0M4tIK9tEDEfJl7",
@@ -230,6 +237,7 @@ def transcript_segments(sequence_locations, gene_examples):
             "elementGenomicEnd": sequence_locations[3],
             "coverage": BreakpointCoverage(fragmentCoverage=100),
             "anchoredReads": AnchoredReads(reads=85),
+            "type": "TranscriptSegmentElement",
         },
         {
             "type": "TranscriptSegmentElement",
@@ -417,7 +425,7 @@ def test_functional_domain(functional_domains, gene_examples):
     check_validation_error(exc_info, msg)
 
 
-def test_transcript_segment_element(transcript_segments):
+def test_transcript_segment_element(transcript_segments, sequence_locations):
     """Test TranscriptSegmentElement object initializes correctly"""
     test_element = TranscriptSegmentElement(**transcript_segments[0])
     assert test_element.transcript == "refseq:NM_152263.3"
@@ -512,10 +520,10 @@ def test_transcript_segment_element(transcript_segments):
     with pytest.raises(ValidationError) as exc_info:
         assert TranscriptSegmentElement(
             element_type="templated_sequence",
-            transcript="NM_152263.3",
+            transcript="refseq:NM_152263.3",
             strand="-1",
-            exonStart="1",
-            exonStartOffset="-9",
+            exonStart=1,
+            exonStartOffset=-9,
             gene={
                 "primaryCoding": {"id": "test:1", "code": "test:1", "system": "test"},
                 "name": "G1",
@@ -528,28 +536,16 @@ def test_transcript_segment_element(transcript_segments):
     with pytest.raises(ValidationError) as exc_info:
         assert TranscriptSegmentElement(
             type="TranscriptSegmentElement",
-            transcript="NM_152263.3",
+            transcript="refseq:NM_152263.3",
             strand="-1",
-            exonStartOffset="-9",
-            exonEndOffset="7",
+            exonStartOffset=-9,
+            exonEndOffset=7,
             gene={
                 "primaryCoding": {"id": "test:1", "code": "test:1", "system": "test"},
                 "name": "G1",
             },
-            elementGenomicStart={
-                "location": {
-                    "species_id": "taxonomy:9606",
-                    "chr": "12",
-                    "interval": {"start": "p12.1", "end": "p12.2"},
-                }
-            },
-            elementGenomicEnd={
-                "location": {
-                    "species_id": "taxonomy:9606",
-                    "chr": "12",
-                    "interval": {"start": "p12.2", "end": "p12.2"},
-                }
-            },
+            elementGenomicStart=sequence_locations[0],
+            elementGenomicEnd=sequence_locations[1],
         )
     msg = "Value error, Must give values for either `exonStart`, `exonEnd`, or both"
     check_validation_error(exc_info, msg)
@@ -781,7 +777,7 @@ def test_event():
         CausativeEvent(eventType="combination")
 
 
-def test_regulatory_element(regulatory_elements, gene_examples):
+def test_regulatory_element(regulatory_elements, gene_examples, sequence_locations):
     """Test RegulatoryElement object initializes correctly"""
     test_reg_elmt = RegulatoryElement(**regulatory_elements[0])
     assert test_reg_elmt.regulatoryClass.value == "promoter"
@@ -803,8 +799,20 @@ def test_regulatory_element(regulatory_elements, gene_examples):
         == "Value error, Must set 1 of {`featureId`, `associatedGene`} and/or `featureLocation`"
     )
 
+    # Require chromosomal build
+    with pytest.raises(ValidationError) as exc_info:
+        RegulatoryElement(
+            regulatoryClass="enhancer",
+            associatedGene=gene_examples[0],
+            featureLocation=sequence_locations[6],
+        )
+    assert (
+        exc_info.value.errors()[0]["msg"]
+        == "Value error, `name` must be a RefSeq chromosomal accession that starts with `NC_`"
+    )
 
-def test_fusion(
+
+def test_fusion_itd(
     functional_domains,
     transcript_segments,
     templated_sequence_elements,
@@ -813,8 +821,8 @@ def test_fusion(
     regulatory_elements,
     unknown_element,
 ):
-    """Test that Fusion object initializes correctly"""
-    # test valid object
+    """Test that Fusion and ITD object initializes correctly"""
+    # test valid Fusion object
     fusion = CategoricalFusion(
         readingFramePreserved=True,
         criticalFunctionalDomains=[functional_domains[0]],
@@ -925,8 +933,18 @@ def test_fusion(
     msg = "Value error, 5' TranscriptSegmentElement fusion partner must contain ending exon position"
     check_validation_error(exc_info, msg)
 
+    # Test valid ITD
+    itd = InternalTandemDuplication(
+        readingFramePreserved=True,
+        criticalFunctionalDomains=[functional_domains[0]],
+        structure=[transcript_segments[1], transcript_segments[1]],
+        regulatoryElement=regulatory_elements[0],
+    )
+    assert itd.structure[0].transcript == "refseq:NM_034348.3"
+    assert itd.structure[1].transcript == "refseq:NM_034348.3"
 
-def test_fusion_element_count(
+
+def test_fusion_itd_element_count(
     functional_domains,
     regulatory_elements,
     unknown_element,
@@ -934,19 +952,32 @@ def test_fusion_element_count(
     transcript_segments,
     gene_examples,
 ):
-    """Test fusion element count requirements."""
+    """Test fusion element and ITD element count requirements."""
     # elements are mandatory
     with pytest.raises(ValidationError) as exc_info:
         assert AssayedFusion(
+            structure=[],
             functionalDomains=[functional_domains[1]],
-            causativeEvent="rearrangement",
-            regulatoryElement=[regulatory_elements[0]],
+            causativeEvent={"eventType": "rearrangement"},
+            regulatoryElement=regulatory_elements[0],
         )
     element_ct_msg = (
-        "Value error, Fusions must contain >= 2 structural elements, or >=1 structural element "
+        "Value error, Fusions must contain >= 2 structural elements, or >= 1 structural element "
         "and a regulatory element"
     )
     check_validation_error(exc_info, element_ct_msg)
+    with pytest.raises(ValidationError) as exc_info:
+        assert InternalTandemDuplication(
+            functionalDomains=[functional_domains[1]],
+            causativeEvent={"eventType": "rearrangement"},
+            regulatoryElement=regulatory_elements[0],
+            structure=[],
+        )
+    element_ct_msg_itd = (
+        "Value error, ITDs must contain >= 2 structural elements, or >= 1 structural element "
+        "and a regulatory element"
+    )
+    check_validation_error(exc_info, element_ct_msg_itd)
 
     # must have >= 2 elements + regulatory elements
     with pytest.raises(ValidationError) as exc_info:
@@ -955,7 +986,7 @@ def test_fusion_element_count(
             causativeEvent={
                 "type": "CausativeEvent",
                 "eventType": "rearrangement",
-                "event_description": "chr2:g.pter_8,247,756::chr11:g.15,825,273_cen_qter (der11) and chr11:g.pter_15,825,272::chr2:g.8,247,757_cen_qter (der2)",
+                "eventDescription": "chr2:g.pter_8,247,756::chr11:g.15,825,273_cen_qter (der11) and chr11:g.pter_15,825,272::chr2:g.8,247,757_cen_qter (der2)",
             },
             assay={
                 "type": "Assay",
@@ -967,12 +998,18 @@ def test_fusion_element_count(
         )
     check_validation_error(exc_info, element_ct_msg)
 
-    # unique gene requirements
+    # unique gene requirements for fusions
     uq_gene_error_msg = "Value error, Fusions must form a chimeric transcript from two or more genes, or a novel interaction between a rearranged regulatory element with the expressed product of a partner gene."
     with pytest.raises(ValidationError) as exc_info:
         assert CategoricalFusion(structure=[gene_elements[0], gene_elements[0]])
     check_validation_error(exc_info, uq_gene_error_msg)
 
+    # same gene requirement for ITDs
+    same_gene_error_msg = "Value error, ITDs must be formed from only one unique gene."
+    with pytest.raises(ValidationError) as exc_info:
+        assert InternalTandemDuplication(structure=[gene_elements[0], gene_elements[1]])
+    check_validation_error(exc_info, same_gene_error_msg)
+
     with pytest.raises(ValidationError) as exc_info:
         assert CategoricalFusion(structure=[gene_elements[1], transcript_segments[0]])
     check_validation_error(exc_info, uq_gene_error_msg)
@@ -1030,14 +1067,25 @@ def test_fusion_element_count(
         )
 
 
-def test_fusion_abstraction_validator(transcript_segments, linkers):
-    """Test that instantiation of abstract fusion fails."""
+def test_abstraction_validator(transcript_segments, linkers):
+    """Test that instantiation of AbstractTranscriptStructural variant
+    and AbstractFusion fails.
+    """
+    # can't create base AbstractTranscriptStructuralVariant
+    with pytest.raises(
+        TypeError,
+        match="Cannot instantiate AbstractTranscriptStructuralVariant abstract class",
+    ):
+        AbstractTranscriptStructuralVariant(
+            structure=[transcript_segments[2], linkers[0]]
+        )
+
     # can't create base fusion
-    with pytest.raises(ValidationError) as exc_info:
-        assert AbstractFusion(structure=[transcript_segments[2], linkers[0]])
-    check_validation_error(
-        exc_info, "Value error, Cannot instantiate Fusion abstract class"
-    )
+    with pytest.raises(
+        TypeError,
+        match="Cannot instantiate Fusion abstract class",
+    ):
+        AbstractFusion(structure=[transcript_segments[2], linkers[0]])
 
 
 def test_file_examples():
@@ -1061,6 +1109,7 @@ def test_model_examples():
         CausativeEvent,
         AssayedFusion,
         CategoricalFusion,
+        InternalTandemDuplication,
     ]
     for model in models:
         schema = model.model_config["json_schema_extra"]
@@ -1078,17 +1127,27 @@ def test_save_cache():
     categorical_fusion = CategoricalFusion(
         **CategoricalFusion.model_config["json_schema_extra"]["example"]
     )
+    itd = InternalTandemDuplication(
+        **InternalTandemDuplication.model_config["json_schema_extra"]["example"]
+    )
 
     # Test AssayedFusion
     save_fusions_cache(
-        fusions_list=[assayed_fusion],
+        variants_list=[assayed_fusion],
         cache_name="assayed_cache_test.pkl",
     )
     assert Path.exists(config.data_root / "assayed_cache_test.pkl")
 
     # Test CategoricalFusion
     save_fusions_cache(
-        fusions_list=[categorical_fusion],
+        variants_list=[categorical_fusion],
         cache_name="categorical_cache_test.pkl",
     )
     assert Path.exists(config.data_root / "categorical_cache_test.pkl")
+
+    # Test ITD
+    save_fusions_cache(
+        variants_list=[itd],
+        cache_name="itd_cache_test.pkl",
+    )
+    assert Path.exists(config.data_root / "itd_cache_test.pkl")
